package com.jdcloud.media.demo.common.network;

import android.app.AlertDialog;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Handler;
import android.os.Looper;
import android.telephony.TelephonyManager;
import android.text.TextUtils;

import com.facebook.stetho.okhttp3.StethoInterceptor;
import com.google.gson.Gson;
import com.jdcloud.media.demo.base.BaseRequest;
import com.jdcloud.media.demo.utils.LogUtil;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import okhttp3.Cache;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.logging.HttpLoggingInterceptor;

import static android.app.AlertDialog.*;

public class OkHttpUtil {
    private static final long DEFAULT_READ_TIMEOUT_MILLIS = 10 * 1000;
    private static final long DEFAULT_WRITE_TIMEOUT_MILLIS = 10 * 1000;
    private static final long DEFAULT_CONNECT_TIMEOUT_MILLIS = 10 * 1000;
    private static final long HTTP_RESPONSE_DISK_CACHE_MAX_SIZE = 10 * 1024 * 1024;
    private static final String TAG = OkHttpUtil.class.getSimpleName();
    private Handler mHandler;
    private OkHttpClient mClient;

    public static final int ERRORCODE_300=300;//请求没成功
    public static final int ERRORCODE_400=400;//请求结果为空
    public static final int ERRORCODE_500=500;//解析出现错误
    private static volatile OkHttpUtil INSTANCE;
    private static boolean isShowing;

    public static OkHttpUtil getInstance() {
        if (INSTANCE == null) {
            synchronized (OkHttpUtil.class) {
                if (INSTANCE == null) {
                    INSTANCE = new OkHttpUtil();
                }
            }
        }
        return INSTANCE;
    }

    public void initHttp() {
        HttpLoggingInterceptor loggingInterceptor = new HttpLoggingInterceptor();
        loggingInterceptor.setLevel(HttpLoggingInterceptor.Level.BODY);
        mClient = new OkHttpClient.Builder()
                .readTimeout(DEFAULT_READ_TIMEOUT_MILLIS, TimeUnit.MILLISECONDS)
                .writeTimeout(DEFAULT_WRITE_TIMEOUT_MILLIS, TimeUnit.MILLISECONDS)
                .connectTimeout(DEFAULT_CONNECT_TIMEOUT_MILLIS, TimeUnit.MILLISECONDS)
                .addNetworkInterceptor(new StethoInterceptor())
                .addInterceptor(loggingInterceptor)
                .build();
        mHandler = new Handler(Looper.getMainLooper());
    }

    public void setCache(Context appContext) {
        final File baseDir = appContext.getApplicationContext().getCacheDir();
        if (baseDir != null) {
            final File cacheDir = new File(baseDir, "jdc_HttpCache");
            mClient.newBuilder().cache((new Cache(cacheDir, HTTP_RESPONSE_DISK_CACHE_MAX_SIZE)));
        }
    }

    public void doBaseRequest(String url, String tag, ResponseCallback<? extends BaseResponse> onResponse) {
        BaseRequest br = new BaseRequest();
        Map<String, String> params = new HashMap<>();
        params.put("appkey", br.appkey);
        params.put("tm", String.valueOf(br.tm));
        params.put("auth", br.auth);
        params.put("appId", br.appId);
        doRequest(url, tag, params, HttpRequest.Method.POST, onResponse);
    }

    public void doRequestByPost(String url, String tag, Map<String, String> params, ResponseCallback<? extends BaseResponse> onResponse) {
        doRequest(url, tag, params, HttpRequest.Method.POST, onResponse);
    }


    public void doRequest(String url, final String tag, final Map<String, String> params, int method, final ResponseCallback<? extends BaseResponse> onResponse) {
        HttpRequest okHttpRequest = new HttpRequest(method);
        if(params!=null && !params.isEmpty()){
            okHttpRequest.putParams(params);
        }
        Request.Builder requestBuild;
        if (okHttpRequest.isPost()) {
            requestBuild = new Request.Builder().url(url)
                    .tag(tag).put(okHttpRequest.getRequestBody());
        } else {
            requestBuild = new Request.Builder().url(url + okHttpRequest.getGetBody())
                    .tag(tag).get();
        }
        LogUtil.d(TAG, tag+"请求地址:" + url+"\nParams:" + params);
        mClient.newCall(requestBuild.build()).enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                LogUtil.e(TAG, "onFailure:" + e.toString());
                if (call.isCanceled()) {
                    onError(tag,onResponse, ERRORCODE_300);
                } else {
                    onError(tag,onResponse, ERRORCODE_300);
                }
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                try {
                    final String responseStr = response.body().string();
                    LogUtil.d(TAG, tag+"请求返回为：\n"+responseStr);
                    if (response != null && !TextUtils.isEmpty(responseStr) && response.isSuccessful()) {
                        onSuccess(tag, onResponse, responseStr);
                    } else {
                        onError(tag,onResponse, ERRORCODE_400);
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                    onError(tag,onResponse, ERRORCODE_500);
                }
            }
        });
    }


    private void onError(final String tag,final ResponseCallback onResponse,final int errorCode) {
        mHandler.post(new Runnable() {
            @Override
            public void run() {
                if (onResponse != null) {
                    onResponse.onError(tag,errorCode);
                }
            }
        });
    }


    private void onSuccess(final String tag, final ResponseCallback onResponse, final String responseStr) {
        mHandler.post(new Runnable() {
            @Override
            public void run() {
                if (onResponse != null) {
                        Gson gson = new Gson();
                        BaseResponse response = null;
                        try {
                            response = gson.fromJson(responseStr, BaseResponse.class);
                            onResponse.onSuccess(response);
                        } catch (Exception e) {
                            e.printStackTrace();
                            LogUtil.e(TAG,tag+"请求出现解析错误："+responseStr);
                            onResponse.onError(tag,OkHttpUtil.ERRORCODE_500);
                        }
                }
            }
        });
    }


    public void cancelTag(Object tag) {
        for (Call call : mClient.dispatcher().queuedCalls()) {
            if (tag.equals(call.request().tag())) {
                call.cancel();
                LogUtil.d(TAG, "cancelTag:" + tag);
            }
        }
        for (Call call : mClient.dispatcher().runningCalls()) {
            if (tag.equals(call.request().tag())) {
                call.cancel();
                LogUtil.d(TAG, "cancelTag:" + tag);
            }
        }
    }

    public void cancelAll() {
        mClient.dispatcher().cancelAll();
    }

    /**
     * @param context
     * @return 是否有可用网络
     */
    public static boolean checkConnect(final Context context) {
        ConnectivityManager conManager = (ConnectivityManager)context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo network = conManager.getActiveNetworkInfo();
        if (network != null) {
            if(conManager.getActiveNetworkInfo().isAvailable()){
                return true;
            }
        }
        if(isShowing){
            return false;
        }
        Builder builder = new Builder(context);
        builder.setTitle("提示信息").setMessage("未发现网络连接,是否进行设置?").setPositiveButton("设置", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                // TODO Auto-generated method stub
                Intent intent = null;
                //3.0之前和之后API不同，需要判断
                if (android.os.Build.VERSION.SDK_INT > 10) {
                    intent = new Intent(android.provider.Settings.ACTION_WIRELESS_SETTINGS);
                } else {
                    intent = new Intent();
                    ComponentName component = new ComponentName("com.android.settings", "com.android.settings.WirelessSettings");
                    intent.setComponent(component);
                    intent.setAction("android.intent.action.VIEW");
                }
                context.startActivity(intent);
                isShowing=false;
            }
        }).setNegativeButton("取消", new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface dialog, int which) {
                // TODO Auto-generated method stub
                dialog.dismiss();
                isShowing=false;
            }
        }).setCancelable(false).show();
        isShowing=true;
        return false;
    }

    public static boolean isWifi(Context context) {
        ConnectivityManager mgrConn = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        TelephonyManager mgrTel = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
        if(mgrConn.getActiveNetworkInfo() != null && mgrConn.getActiveNetworkInfo().getState() == NetworkInfo.State.CONNECTED){
            if( mgrConn.getActiveNetworkInfo().getType() == ConnectivityManager.TYPE_WIFI){
                return true;
            }
        }
        return false;
    }

    private boolean networkGrant(Context context){
        if(OkHttpUtil.checkConnect(context)){
            if(OkHttpUtil.isWifi(context)){
                return true;
            }else{
                AlertDialog.Builder builder = new AlertDialog.Builder(context);
                builder.setTitle("提示信息").setMessage("当前网络环境不是WIFI，是否继续播放?").setPositiveButton("继续", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        // TODO Auto-generated method stub
                    }
                }).setNegativeButton("取消", new DialogInterface.OnClickListener() {

                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        // TODO Auto-generated method stub
                        dialog.dismiss();
                    }
                }).setCancelable(false).show();
            }
        }
        return false;
    }


}
